/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2008 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#ifndef TAAG_H_
#define TAAG_H_

/* including MPICH-G2 */
#include <mpi.h>
#include <taag_basic.h>
#include <taag_errno.h>

#define TAAG_ROOT 0  //the rank of the root group (as well as the rank of the root process)

#define TAAG_WAN_LEVEL  0
#define TAAG_LAN_LEVEL  1
#define TAAG_HOST_LEVEL 2

#ifndef TAAG_CALLS_PROTOTYPES
#define TAAG_CALLS_PROTOTYPES
/* Definition of the prototypes of the general TAAG calls */
int TAAG_Init(char *);
int TAAG_Initialized(int *);
int TAAG_Free();

/* Definition of the prototypes of the group-related TAAG calls */
int TAAG_Group_number(int *);
int TAAG_Group_graph(int, int *);

int TAAG_Group_rank(int, int *);

int TAAG_Group_size(int, int *);
int TAAG_Group_members(int, int , int *);

int TAAG_Group_MPIGroup(int, int *, int, int *, MPI_Group *);

int TAAG_Group_degree(int, int, int *);
int TAAG_Group_neighbours(int, int, int, int *);

int TAAG_Group_distance(int, int, int*);
int TAAG_Group_way(int, int, int, int*);

int TAAG_Group_element(int, int, int *);

/* Definition of the prototypes of the tree-related TAAG calls */
int TAAG_Tree_isTree(int *);
int TAAG_Tree_depth(int *);

int TAAG_Tree_level(int, int *);
int TAAG_Tree_width(int, int, int *);
int TAAG_Tree_children(int, int, int, int *);
int TAAG_Tree_parent(int, int *);

int TAAG_Tree_root(int *);
int TAAG_Tree_isLeaf(int, int *);
int TAAG_Tree_isLocalManager(int, int *);

/* Definition of the prototypes of the ring-related TAAG calls */
int TAAG_Ring_isRing(int *);
int TAAG_Ring_left(int, int *);
int TAAG_Ring_right(int, int *);

/* Definition of the prototypes of the HW-topology-related TAAG calls */
int TAAG_Topology_given(int *);
int TAAG_CPU_number(int *);

int TAAG_Host_number(int *);
int TAAG_Host_properties(int, int *, int *);
int TAAG_Host_processes(int, int, int *);
int TAAG_Host_latency(int, int, double *);
int TAAG_Host_address(int, char *);

int TAAG_LAN_number(int *);
int TAAG_LAN_properties(int, int *, int *, int *);
//int TAAG_LAN_host(int, int, char *);
int TAAG_LAN_processes(int, int, int *);

int TAAG_Comm_level(int, int, int *);

#endif /* TAAG_CALLS_PROTOTYPES */

#endif /* TAAG_H_ */
