/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2008 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#include <taagimpl.h>

/***********************************************************************
 * TAAG_Ring_isRing indicates whether given execution plan (ﬁle)
 * describes a ring structure. It returns a ﬂag as either logical
 * true (1) or false(0).
 ***********************************************************************/
int TAAG_Ring_isRing(int *flag) {
	if (_taag_initialized == TAAG_FALSE) return (TAAG_ERR_INIT);

	if (SCHEMA_RING != _taag_getType()) {
			*flag = TAAG_FALSE;
		}
		else {
			*flag = TAAG_TRUE;
		}
	return (TAAG_SUCCESS);
}

/***********************************************************************
 * TAAG_Ring_left requires a group rank as input and it returns the rank
 * of the left neighbor group of the given group.
 ***********************************************************************/
int TAAG_Ring_left(int grpRank1, int *grpRank2) {
	int nrGroups;
	int nrGroupsOnLevel;

	if (_taag_initialized == TAAG_FALSE) return (TAAG_ERR_INIT);
	if (SCHEMA_RING != _taag_getType()) return (TAAG_ERR_SCHEMA);

	nrGroups = _taag_getNumberOfGroups();
	if (grpRank1 < 0 || grpRank1 >= nrGroups) return (TAAG_ERR_GRANK);

	TAAG_Group_degree(grpRank1, 1, &nrGroupsOnLevel);

	if (nrGroupsOnLevel != 2) return (TAAG_ERR_UNKNOWN);

	int neighbours[2];
	TAAG_Group_neighbours(grpRank1, 1, 2, neighbours);

    if (grpRank1 == 0) {
    	if (neighbours[1] == nrGroups -1) *grpRank2 = nrGroups-1;
    	else return (TAAG_ERR_UNKNOWN);
    }
    else if (grpRank1 == nrGroups-1) {
    	if (neighbours[1] == grpRank1 - 1) *grpRank2 = grpRank1 -1;
    	else return (TAAG_ERR_UNKNOWN);
    }
    else {
    	if (neighbours[0] == grpRank1 - 1) *grpRank2 = grpRank1 -1;
    	else return (TAAG_ERR_UNKNOWN);
    }

	return (TAAG_SUCCESS);
}

/***********************************************************************
 * TAAG_Ring_right requires a group rank as input and it returns the rank
 * of the right neighbor group of the given group.
 ***********************************************************************/
int TAAG_Ring_right(int grpRank1, int *grpRank2) {
	int nrGroups;
	int nrGroupsOnLevel;

	if (_taag_initialized == TAAG_FALSE) return (TAAG_ERR_INIT);
	if (SCHEMA_RING != _taag_getType()) return (TAAG_ERR_SCHEMA);

	nrGroups = _taag_getNumberOfGroups();
	if (grpRank1 < 0 || grpRank1 >= nrGroups) return (TAAG_ERR_GRANK);

	TAAG_Group_degree(grpRank1, 1, &nrGroupsOnLevel);

	if (nrGroupsOnLevel != 2) return (TAAG_ERR_UNKNOWN);

	int neighbours[2];
	TAAG_Group_neighbours(grpRank1, 1, 2, neighbours);

    if (grpRank1 == nrGroups -1) {
    	if (neighbours[0] == 0) *grpRank2 = 0;
    	else return (TAAG_ERR_UNKNOWN);
    }
    else if (grpRank1 == 0) {
    	if (neighbours[0] == grpRank1 + 1) *grpRank2 = grpRank1 + 1;
    	else return (TAAG_ERR_UNKNOWN);
    }
    else {
    	if (neighbours[1] == grpRank1 + 1) *grpRank2 = grpRank1 + 1;
    	else return (TAAG_ERR_UNKNOWN);
    }

	return (TAAG_SUCCESS);
}

