/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2010 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#include "HostData.h"
#define ROUNDING_CPU_FRACTION 0.5  //this helps to schedule process(es) to partially occupied CPUs

HostData::HostData(int _id, char *_hostname, double _CPUForecast, int _CPUcount) {
	processesPerCPU = 1;
	id = _id;
	strcpy(hostname, _hostname);
	CPUForecast = _CPUForecast;
	CPUcount = _CPUcount;
	plannedLoad = 0;
	addressOfHighestCluster = NULL;
	addressOfLANCluster = NULL;
}

HostData::HostData(int _id, char *_hostname, double _CPUForecast, int _CPUcount, int _factor) {
	if (_factor>0 && _factor<20) {
		processesPerCPU = _factor;
	}
	else {
		processesPerCPU = 1;
	}
	id = _id;
	strcpy(hostname, _hostname);
	CPUForecast = _CPUForecast;
	CPUcount = _CPUcount;
	plannedLoad = 0;
	addressOfHighestCluster = NULL;
	addressOfLANCluster = NULL;
}

HostData::~HostData() {
	// TODO Auto-generated destructor stub
}

int HostData::getHostId() {
	return id;
}

char *HostData::getHostname() {
	return hostname;
}

double HostData::getCPUForecast() {
	return CPUForecast;
}

int HostData::getCPUcount() {
	return CPUcount;
}

int HostData::availableProcsCapacity() {
	int capacity = (int)(CPUForecast+ROUNDING_CPU_FRACTION)*processesPerCPU-plannedLoad;
	if (capacity <= 0) return 0;
	return capacity;
}

void HostData::increaseLoad(int nrOfProcs) {
	plannedLoad+=nrOfProcs;
}

void HostData::decreaseLoad(int nrOfProcs) {
	plannedLoad-=nrOfProcs;
	if (plannedLoad<0) {plannedLoad=0;}
}

int HostData::getLoad() {
	return plannedLoad;
}

void HostData::clearLoad() {
	plannedLoad = 0;
}

void HostData::setAddressOfHighestCluster(void *address) {
	addressOfHighestCluster = address;
}

void *HostData::getAddressOfHighestCluster() {
	return addressOfHighestCluster;
}

void HostData::setAddressOfLANCluster(void *address) {
	addressOfLANCluster = address;
}

void *HostData::getAddressOfLANCluster() {
	return addressOfLANCluster;
}

