/*
 * $Id$
 * Author: Karoly Bosa
 *
 * (C) 2010 by Research Institute for Symbolic Computation, Linz, Austria
 * All rights reserved. See COPYRIGHT in top-level directory.
 */

#define NWSAPI_SHORTNAMES
#define HOW_MANY 20 //how many nws measurements used at most for the forecast
#define LATENCY_PERIOD 30 //how often latency is checked

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vector>

#include <nws_api.h>

#include <taag_basic.h>
#include <taag_errno.h>

#include "HostData.h"
#include "ListOfHosts.h"
#include "LatencyMatrix.h"
#include "SensorData.h"

using namespace std;

static char name[TAAG_MAX_HOSTNAME_STRING];
static char series[TAAG_MAX_HOSTNAME_STRING];
static char actName[TAAG_MAX_HOSTNAME_STRING];
static char member1[TAAG_MAX_HOSTNAME_STRING];
static char member2[TAAG_MAX_HOSTNAME_STRING];

static int port;
static HostData *host;
static SensorData *sd;
static int hostCounter = 0;
static unsigned int howMany;

static HostSpec NS;
static vector<SensorData *> sensorNames;

static ForecastState *state;
static HostSpec sensor;
static SeriesSpec spec;

static bool successfulMeasurements = true;
static bool successfulActivityStart = true;

//static char nwsNameServer[TAAG_MAX_HOSTNAME_STRING];
//static unsigned int nwsNameServerPort;


int _nws_setNameServer(char *_nwsNameServer, unsigned int _nwsNameServerPort) {
	//strcpy(nwsNameServer, _nwsNameServer);
	//nwsNameServerPort = _nwsNameServerPort;

	/* let's instruct the interface to use the right nameserver */
	NS = *MakeHostSpec(_nwsNameServer, _nwsNameServerPort);
	UseNameServer(&NS);
	return TAAG_SUCCESS;
}

int _nws_getHosts(ListOfHosts *hostList, int factor) {
	char filter[256];
	ObjectSet *objs;
	Object current;
	NwsAttribute attr;

	/* create the right filter */
	sprintf(filter, "hostType=sensor");

	/* query the NS */
	GetObjects(filter, objs);

	Measurement meas[HOW_MANY];
	ForecastCollection forecasts[HOW_MANY];

	/* now we can browse though the objects*/
	for (current = NextObject(*objs, NULL); current != NULL; current = NextObject(*objs, current)) {
		/* now we can browse though the attribute*/
		for (attr = NextNwsAttribute(current, NULL); attr != NULL; attr = NextNwsAttribute(current, attr)) {
			/* do some pretty printing */
			char *tmp = NwsAttributeName_r(attr);
			char *tmp1 = NwsAttributeValue_r(attr);
			if (!strcmp(tmp, "name")) {
				strcpy(name, tmp1);
				if(strchr(name, ':') != NULL) {
					*(strchr(name, ':'))='\0';
				}
				port = atoi(strchr(tmp1, ':')+1);
			}
			else if (!strcmp(tmp, "CPUcount")) {
				sd = new SensorData(name, port, atoi(tmp1));
				sensorNames.push_back(sd);
			}
			free(tmp);
			free(tmp1);
		} //for
	} //for
	FreeObjectSet(objs);

	for (unsigned int i=0; i < sensorNames.size(); i++) {
		sprintf(series,"%s.availableCpu.0", sensorNames[i]->getName());
		state = NewForecastState();
		if (!GetMeasurements(series, NWSAPI_BEGINNING_OF_TIME, meas, HOW_MANY, &howMany))
		{
			fprintf(stderr, "Error: Failed to retrieve CPU availability measurement(s).\n");
			fprintf(stderr, "Perhaps not all NWS sensors are healthy. Please check you NWS configuration!\n");
			return TAAG_ERR_NWS;
		}
		// we have howMany measurements: generate the forecasts now.
		UpdateForecastState(state, meas, howMany, forecasts, howMany);

		/* we use only the MAE forecasts here */
		//printf("%s: %.2f\n", series, forecasts[0].forecasts[NWSAPI_MAE_FORECAST].forecast);  //BK_TEST
		host = new HostData(hostCounter++, sensorNames[i]->getHostName(), forecasts[0].forecasts[NWSAPI_MAE_FORECAST].forecast, sensorNames[i]->getCPUcount(), factor);
		hostList->addHost(host);

		/* free the memory associated with the forecast state */
		FreeForecastState(&state);
	}
	return TAAG_SUCCESS;
}


int _nws_getLatencies(int nrOfHosts, LatencyMatrix *matrix) {
	/* how many measurement we retrieve */
	Measurement meas[HOW_MANY];
	ForecastCollection forecasts[HOW_MANY];

	for (unsigned int i=0; i<sensorNames.size(); i++) {
		for (unsigned int j = i+1; j<sensorNames.size(); j++) {

			/* get the series name: the right way is to query the nameserver to find
			 * the right name of the series. For simplicity we build one here,
			 * assuming that there is indeed such a series registered with the
			 * nameserver */
			//full hostname with port number is required!!!
			spec = *MakeSeriesSpec(sensorNames[i]->getName(), sensorNames[j]->getName(), "latencyTcp");
			strcpy(series, SeriesName(&spec));

			//printf("Series name: %s\n", series); //BK_TEST
			/* initialize the forecast state */
			state = NewForecastState();

			/* get the measurements */
			//if (!GetMeasurements("worker2.local.testbed:8060.latencyTcp.32.16.256.worker1.local.testbed:8060", NWSAPI_BEGINNING_OF_TIME, meas, HOW_MANY, &howMany))
			if (!GetMeasurements(series, NWSAPI_BEGINNING_OF_TIME, meas, HOW_MANY, &howMany))
			{
				sensor = *MakeHostSpec(sensorNames[i]->getHostName(), sensorNames[i]->getPort());
				sprintf(actName, "LatencyTest%d_%d", i, j);
				sprintf(member1, "member:%s", sensorNames[i]->getName());
				sprintf(member2, "member:%s", sensorNames[j]->getName());
				const char *options[] = {	member1,  //it works with single host names, too
											member2,
											"period:30"};
				//sprintf(options[2], "period:%d", LATENCY_PERIOD);

				if (!StartActivity(&sensor, actName, "clique", "tcpMessageMonitor", options, 3)) {
					fprintf(stderr, "\nError:Unsuccessful activity start!\n");
					successfulActivityStart = false;
				}
				else {
					fprintf(stderr, "\n\"tcpMessageMonitor\" is started between %s and %s\n", sensorNames[i]->getHostName(), sensorNames[j]->getHostName());
				}
				successfulMeasurements = false;
				continue;
			}
			/* we have howMany measurements: generate the forecasts now. */
			UpdateForecastState(state, meas, howMany, forecasts, howMany);


			/* we use only the MAE forecasts here */
			//printf(" forecast %.4f of %d and %d\n", forecasts[0].forecasts[NWSAPI_MAE_FORECAST].forecast, i, j); //BK_TEST
			matrix->initValue(forecasts[0].forecasts[NWSAPI_MAE_FORECAST].forecast, forecasts[0].forecasts[NWSAPI_MAE_FORECAST].forecast, i, j);

			/* free the memory associated with the forecast state */

			FreeForecastState(&state);

		}  //for j
	} //for i

	while (!sensorNames.empty()) {
		sd = sensorNames.back();
		sensorNames.pop_back();
		delete sd;
	}
	if (successfulMeasurements) {
		return TAAG_SUCCESS;
	}
	else {
		fprintf(stderr, "\nError: Failed to retrieve latency measurements between some hosts.\n");
		fprintf(stderr, "Perhaps not all possible connections are monitored by NWS.\n");
		if (successfulActivityStart) {
			fprintf(stderr, "\nTAAG system has already started the required \"tcpMessageMonitor\" activities.\n");
			fprintf(stderr, "Please wait 30 seconds before you restart the TAAG scheduler!\n\n.");
		}
		else {
			fprintf(stderr, "\nTAAG system was not able to start the required \"tcpMessageMonitor\" activities either.\n");
			fprintf(stderr, "Please check you NWS configuration!\n\n");
		}
		return TAAG_ERR_NWS;
	}

}

